@@ -462,41 +462,62 @@ func NewBooleanQuery() *BooleanQuery {
 	return q
 }
 
-// Must specifies a query which must match.
-func (q *BooleanQuery) Must(query Query) *BooleanQuery {
-	switch val := query.(type) {
-	case ConjunctionQuery:
-		q.data.Must = &val
-	case *ConjunctionQuery:
-		q.data.Must = val
+// Must specifies queries which must match.
+func (q *BooleanQuery) Must(queries ...Query) *BooleanQuery {
+	switch len(queries) {
+	case 0:
+		q.data.Must = nil
+	case 1:
+		switch val := queries[0].(type) {
+		case ConjunctionQuery:
+			q.data.Must = &val
+		case *ConjunctionQuery:
+			q.data.Must = val
+		default:
+			q.data.Must = NewConjunctionQuery(val)
+		}
 	default:
-		q.data.Must = NewConjunctionQuery(val)
+		q.data.Must = NewConjunctionQuery(queries...)
 	}
 	return q
 }
 
-// Should specifies a query which should match.
-func (q *BooleanQuery) Should(query Query) *BooleanQuery {
-	switch val := query.(type) {
-	case DisjunctionQuery:
-		q.data.Should = &val
-	case *DisjunctionQuery:
-		q.data.Should = val
+// Should specifies queries which should match.
+func (q *BooleanQuery) Should(queries ...Query) *BooleanQuery {
+	switch len(queries) {
+	case 0:
+		q.data.Should = nil
+	case 1:
+		switch val := queries[0].(type) {
+		case DisjunctionQuery:
+			q.data.Should = &val
+		case *DisjunctionQuery:
+			q.data.Should = val
+		default:
+			q.data.Should = NewDisjunctionQuery(val)
+		}
 	default:
-		q.data.Should = NewDisjunctionQuery(val)
+		q.data.Should = NewDisjunctionQuery(queries...)
 	}
 	return q
 }
 
-// MustNot specifies a query which must not match.
-func (q *BooleanQuery) MustNot(query Query) *BooleanQuery {
-	switch val := query.(type) {
-	case DisjunctionQuery:
-		q.data.MustNot = &val
-	case *DisjunctionQuery:
-		q.data.MustNot = val
+// MustNot specifies queries which must not match.
+func (q *BooleanQuery) MustNot(queries ...Query) *BooleanQuery {
+	switch len(queries) {
+	case 0:
+		q.data.MustNot = nil
+	case 1:
+		switch val := queries[0].(type) {
+		case DisjunctionQuery:
+			q.data.MustNot = &val
+		case *DisjunctionQuery:
+			q.data.MustNot = val
+		default:
+			q.data.MustNot = NewDisjunctionQuery(val)
+		}
 	default:
-		q.data.MustNot = NewDisjunctionQuery(val)
+		q.data.MustNot = NewDisjunctionQuery(queries...)
 	}
 	return q
 }
@@ -830,7 +851,6 @@ func NewMatchNoneQuery() *MatchNoneQuery {
 
 // TermRangeQuery represents a search term range query.
 type TermRangeQuery struct {
-	term         string
 	field        *string
 	min          *string
 	inclusiveMin *bool
@@ -842,7 +862,6 @@ type TermRangeQuery struct {
 // MarshalJSON marshal's this query to JSON for the search REST API.
 func (q TermRangeQuery) MarshalJSON() ([]byte, error) {
 	outStruct := &struct {
-		Term         string   `json:"term"`
 		Field        *string  `json:"field,omitempty"`
 		Min          *string  `json:"min,omitempty"`
 		InclusiveMin *bool    `json:"inclusive_min,omitempty"`
@@ -850,7 +869,6 @@ func (q TermRangeQuery) MarshalJSON() ([]byte, error) {
 		InclusiveMax *bool    `json:"inclusive_max,omitempty"`
 		Boost        *float32 `json:"boost,omitempty"`
 	}{
-		Term:         q.term,
 		Field:        q.field,
 		Min:          q.min,
 		InclusiveMin: q.inclusiveMin,
@@ -863,8 +881,8 @@ func (q TermRangeQuery) MarshalJSON() ([]byte, error) {
 }
 
 // NewTermRangeQuery creates a new TermRangeQuery.
-func NewTermRangeQuery(term string) *TermRangeQuery {
-	q := &TermRangeQuery{term: term}
+func NewTermRangeQuery() *TermRangeQuery {
+	q := &TermRangeQuery{}
 	return q
 }
 
