@@ -22,6 +22,7 @@ type httpClientMux struct {
 	backupEpList   []routeEndpoint
 
 	bucket string
+	prod   string
 
 	uuid       string
 	revID      int64
@@ -46,6 +47,7 @@ func newHTTPClientMux(cfg *routeConfig, endpoints httpClientMuxEndpoints, tlsCon
 		backupEpList:   endpoints.backupEpList,
 
 		bucket: cfg.name,
+		prod:   cfg.prod,
 
 		uuid:       cfg.uuid,
 		revID:      cfg.revID,
