@@ -269,6 +269,7 @@ type TransactionGetResult struct {
 	Meta  *TransactionMutableItemMeta
 	Value []byte
 	Cas   Cas
+	Flags uint32
 }
 
 // TransactionGetCallback describes a callback for a completed Get operation.
@@ -333,6 +334,10 @@ type TransactionGetMultiResult struct {
 	// Values is a map with the contents of the documents that were found. The keys of the map are the indexes of the
 	// corresponding specs as they were given in TransactionGetMultiOptions.Specs.
 	Values map[int][]byte
+
+	// Flags is a map with the flags specifying the format of the content for each document that was found. The keys of
+	// the map are the indexes of the corresponding specs as they were given in TransactionGetMultiOptions.Specs.
+	Flags map[int]uint32
 }
 
 // TransactionGetMultiCallback describes a callback for a completed Get operation.
@@ -355,6 +360,7 @@ type TransactionInsertOptions struct {
 	CollectionName string
 	Key            []byte
 	Value          json.RawMessage
+	Flags          uint32
 }
 
 // TransactionStoreCallback describes a callback for a completed Replace operation.
@@ -373,6 +379,7 @@ func (t *Transaction) Insert(opts TransactionInsertOptions, cb TransactionStoreC
 type TransactionReplaceOptions struct {
 	Document *TransactionGetResult
 	Value    json.RawMessage
+	Flags    uint32
 }
 
 // Replace will attempt to replace an existing document.
