@@ -180,11 +180,14 @@ var userAgent = fmt.Sprintf("%v/%v (%v-%v) %v/%v",
 	runtime.Version())
 
 type authRequestClientEnvironment struct {
-	Application string `json:"APPLICATION"`
-	Os          string `json:"OS"`
-	OsVersion   string `json:"OS_VERSION"`
-	OCSPMode    string `json:"OCSP_MODE"`
-	GoVersion   string `json:"GO_VERSION"`
+	Application             string `json:"APPLICATION"`
+	ApplicationPath         string `json:"APPLICATION_PATH"`
+	Os                      string `json:"OS"`
+	OsVersion               string `json:"OS_VERSION"`
+	OCSPMode                string `json:"OCSP_MODE"`
+	GoVersion               string `json:"GO_VERSION"`
+	OAuthType               string `json:"OAUTH_TYPE,omitempty"`
+	CertRevocationCheckMode string `json:"CERT_REVOCATION_CHECK_MODE,omitempty"`
 }
 
 type authRequestData struct {
@@ -203,7 +206,6 @@ type authRequestData struct {
 	BrowserModeRedirectPort string                       `json:"BROWSER_MODE_REDIRECT_PORT,omitempty"`
 	ProofKey                string                       `json:"PROOF_KEY,omitempty"`
 	Token                   string                       `json:"TOKEN,omitempty"`
-	OauthType               string                       `json:"OAUTH_TYPE,omitempty"`
 	Provider                string                       `json:"PROVIDER,omitempty"`
 }
 type authRequest struct {
@@ -271,7 +273,11 @@ func postAuth(
 	if err != nil {
 		return nil, err
 	}
-	defer resp.Body.Close()
+	defer func() {
+		if closeErr := resp.Body.Close(); closeErr != nil {
+			logger.WithContext(ctx).Errorf("failed to close HTTP response body for %v. err: %v", fullURL, closeErr)
+		}
+	}()
 	if resp.StatusCode == http.StatusOK {
 		var respd authResponse
 		err = json.NewDecoder(resp.Body).Decode(&respd)
@@ -351,12 +357,29 @@ func authenticate(
 	}
 
 	headers := getHeaders()
+	// Get the current application path
+	applicationPath, err := os.Executable()
+	if err != nil {
+		applicationPath = "unknown"
+	}
+
+	oauthType := ""
+	switch sc.cfg.Authenticator {
+	case AuthTypeOAuthAuthorizationCode:
+		oauthType = "OAUTH_AUTHORIZATION_CODE"
+	case AuthTypeOAuthClientCredentials:
+		oauthType = "OAUTH_CLIENT_CREDENTIALS"
+	}
+
 	clientEnvironment := authRequestClientEnvironment{
-		Application: sc.cfg.Application,
-		Os:          operatingSystem,
-		OsVersion:   platform,
-		OCSPMode:    sc.cfg.ocspMode(),
-		GoVersion:   runtime.Version(),
+		Application:             sc.cfg.Application,
+		ApplicationPath:         applicationPath,
+		Os:                      operatingSystem,
+		OsVersion:               platform,
+		OCSPMode:                sc.cfg.ocspMode(),
+		GoVersion:               runtime.Version(),
+		OAuthType:               oauthType,
+		CertRevocationCheckMode: sc.cfg.CertRevocationCheckMode.String(),
 	}
 
 	sessionParameters := make(map[string]interface{})
@@ -514,20 +537,15 @@ func createRequestBody(sc *snowflakeConn, sessionParameters map[string]interface
 		}
 	case AuthTypeOAuthAuthorizationCode:
 		logger.WithContext(sc.ctx).Debug("OAuth authorization code")
-		oauthClient, err := newOauthClient(sc.ctx, sc.cfg)
-		if err != nil {
-			return nil, err
-		}
-		token, err := oauthClient.authenticateByOAuthAuthorizationCode()
+		token, err := authenticateByAuthorizationCode(sc)
 		if err != nil {
 			return nil, err
 		}
 		requestMain.LoginName = sc.cfg.User
 		requestMain.Token = token
-		requestMain.OauthType = "OAUTH_AUTHORIZATION_CODE"
 	case AuthTypeOAuthClientCredentials:
 		logger.WithContext(sc.ctx).Debug("OAuth client credentials")
-		oauthClient, err := newOauthClient(sc.ctx, sc.cfg)
+		oauthClient, err := newOauthClient(sc.ctx, sc.cfg, sc)
 		if err != nil {
 			return nil, err
 		}
@@ -537,13 +555,9 @@ func createRequestBody(sc *snowflakeConn, sessionParameters map[string]interface
 		}
 		requestMain.LoginName = sc.cfg.User
 		requestMain.Token = token
-		requestMain.OauthType = "OAUTH_CLIENT_CREDENTIALS"
 	case AuthTypeWorkloadIdentityFederation:
-		if !experimentalAuthEnabled() {
-			return nil, errors.New("workload identity authentication is not ready to use")
-		}
 		logger.WithContext(sc.ctx).Debug("Workload Identity Federation")
-		wifAttestationProvider := createWifAttestationProvider(sc.ctx, sc.cfg)
+		wifAttestationProvider := createWifAttestationProvider(sc.ctx, sc.cfg, sc.telemetry)
 		wifAttestation, err := wifAttestationProvider.getAttestation(sc.cfg.WorkloadIdentityProvider)
 		if err != nil {
 			return nil, err
@@ -566,9 +580,60 @@ func createRequestBody(sc *snowflakeConn, sessionParameters map[string]interface
 	return jsonBody, nil
 }
 
-func experimentalAuthEnabled() bool {
-	val, ok := os.LookupEnv("SF_ENABLE_EXPERIMENTAL_AUTHENTICATION")
-	return ok && strings.EqualFold(val, "true")
+type oauthLockKey struct {
+	tokenRequestURL string
+	user            string
+	flowType        string
+}
+
+func newOAuthAuthorizationCodeLockKey(tokenRequestURL, user string) *oauthLockKey {
+	return &oauthLockKey{
+		tokenRequestURL: tokenRequestURL,
+		user:            user,
+		flowType:        "authorization_code",
+	}
+}
+
+func newRefreshTokenLockKey(tokenRequestURL, user string) *oauthLockKey {
+	return &oauthLockKey{
+		tokenRequestURL: tokenRequestURL,
+		user:            user,
+		flowType:        "refresh_token",
+	}
+}
+
+func (o *oauthLockKey) lockID() string {
+	return o.tokenRequestURL + "|" + o.user + "|" + o.flowType
+}
+
+func authenticateByAuthorizationCode(sc *snowflakeConn) (string, error) {
+	oauthClient, err := newOauthClient(sc.ctx, sc.cfg, sc)
+	if err != nil {
+		return "", err
+	}
+	if !isEligibleForParallelLogin(sc.cfg, sc.cfg.ClientStoreTemporaryCredential) {
+		return oauthClient.authenticateByOAuthAuthorizationCode()
+	}
+
+	lockKey := newOAuthAuthorizationCodeLockKey(oauthClient.tokenURL(), sc.cfg.User)
+	valueAwaiter := valueAwaitHolder.get(lockKey)
+	defer valueAwaiter.resumeOne()
+	token, err := awaitValue(valueAwaiter, func() (string, error) {
+		return credentialsStorage.getCredential(newOAuthAccessTokenSpec(oauthClient.tokenURL(), sc.cfg.User)), nil
+	}, func(s string, err error) bool {
+		return s != ""
+	}, func() string {
+		return ""
+	})
+	if err != nil || token != "" {
+		return token, err
+	}
+	token, err = oauthClient.authenticateByOAuthAuthorizationCode()
+	if err != nil {
+		return "", err
+	}
+	valueAwaiter.done()
+	return token, err
 }
 
 // Generate a JWT token in string given the configuration
@@ -606,20 +671,60 @@ func prepareJWTToken(config *Config) (string, error) {
 	return tokenString, err
 }
 
-// Authenticate with sc.cfg
+type tokenLockKey struct {
+	snowflakeHost string
+	user          string
+	tokenType     string
+}
+
+func newMfaTokenLockKey(snowflakeHost, user string) *tokenLockKey {
+	return &tokenLockKey{
+		snowflakeHost: snowflakeHost,
+		user:          user,
+		tokenType:     "MFA",
+	}
+}
+
+func newIDTokenLockKey(snowflakeHost, user string) *tokenLockKey {
+	return &tokenLockKey{
+		snowflakeHost: snowflakeHost,
+		user:          user,
+		tokenType:     "ID",
+	}
+}
+
+func (m *tokenLockKey) lockID() string {
+	return m.snowflakeHost + "|" + m.user + "|" + m.tokenType
+}
+
 func authenticateWithConfig(sc *snowflakeConn) error {
 	var authData *authResponseMain
 	var samlResponse []byte
 	var proofKey []byte
 	var err error
-	//var consentCacheIdToken = true
+
+	mfaTokenLockKey := newMfaTokenLockKey(sc.cfg.Host, sc.cfg.User)
+	idTokenLockKey := newIDTokenLockKey(sc.cfg.Host, sc.cfg.User)
 
 	if sc.cfg.Authenticator == AuthTypeExternalBrowser || sc.cfg.Authenticator == AuthTypeOAuthAuthorizationCode || sc.cfg.Authenticator == AuthTypeOAuthClientCredentials {
 		if (runtime.GOOS == "windows" || runtime.GOOS == "darwin") && sc.cfg.ClientStoreTemporaryCredential == configBoolNotSet {
 			sc.cfg.ClientStoreTemporaryCredential = ConfigBoolTrue
 		}
-		if sc.cfg.Authenticator == AuthTypeExternalBrowser && sc.cfg.ClientStoreTemporaryCredential == ConfigBoolTrue {
-			sc.cfg.IDToken = credentialsStorage.getCredential(newIDTokenSpec(sc.cfg.Host, sc.cfg.User))
+		if sc.cfg.Authenticator == AuthTypeExternalBrowser {
+			if isEligibleForParallelLogin(sc.cfg, sc.cfg.ClientStoreTemporaryCredential) {
+				valueAwaiter := valueAwaitHolder.get(idTokenLockKey)
+				defer valueAwaiter.resumeOne()
+				sc.cfg.IDToken, _ = awaitValue(valueAwaiter, func() (string, error) {
+					credential := credentialsStorage.getCredential(newIDTokenSpec(sc.cfg.Host, sc.cfg.User))
+					return credential, nil
+				}, func(s string, err error) bool {
+					return s != ""
+				}, func() string {
+					return ""
+				})
+			} else if sc.cfg.ClientStoreTemporaryCredential == ConfigBoolTrue {
+				sc.cfg.IDToken = credentialsStorage.getCredential(newIDTokenSpec(sc.cfg.Host, sc.cfg.User))
+			}
 		}
 		// Disable console login by default
 		if sc.cfg.DisableConsoleLogin == configBoolNotSet {
@@ -631,7 +736,18 @@ func authenticateWithConfig(sc *snowflakeConn) error {
 		if (runtime.GOOS == "windows" || runtime.GOOS == "darwin") && sc.cfg.ClientRequestMfaToken == configBoolNotSet {
 			sc.cfg.ClientRequestMfaToken = ConfigBoolTrue
 		}
-		if sc.cfg.ClientRequestMfaToken == ConfigBoolTrue {
+		if isEligibleForParallelLogin(sc.cfg, sc.cfg.ClientRequestMfaToken) {
+			valueAwaiter := valueAwaitHolder.get(mfaTokenLockKey)
+			defer valueAwaiter.resumeOne()
+			sc.cfg.MfaToken, _ = awaitValue(valueAwaiter, func() (string, error) {
+				credential := credentialsStorage.getCredential(newMfaTokenSpec(sc.cfg.Host, sc.cfg.User))
+				return credential, nil
+			}, func(s string, err error) bool {
+				return s != ""
+			}, func() string {
+				return ""
+			})
+		} else if sc.cfg.ClientRequestMfaToken == ConfigBoolTrue {
 			sc.cfg.MfaToken = credentialsStorage.getCredential(newMfaTokenSpec(sc.cfg.Host, sc.cfg.User))
 		}
 	}
@@ -647,7 +763,6 @@ func authenticateWithConfig(sc *snowflakeConn) error {
 				sc.cfg.Application,
 				sc.cfg.Account,
 				sc.cfg.User,
-				sc.cfg.Password,
 				sc.cfg.ExternalBrowserTimeout,
 				sc.cfg.DisableConsoleLogin)
 			if err != nil {
@@ -667,15 +782,7 @@ func authenticateWithConfig(sc *snowflakeConn) error {
 			credentialsStorage.deleteCredential(newOAuthAccessTokenSpec(sc.cfg.OauthTokenRequestURL, sc.cfg.User))
 
 			if sc.cfg.Authenticator == AuthTypeOAuthAuthorizationCode {
-				var oauthClient *oauthClient
-				if oauthClient, err = newOauthClient(sc.ctx, sc.cfg); err != nil {
-					logger.Warnf("failed to create oauth client. %v", err)
-				} else {
-					if err = oauthClient.refreshToken(); err != nil {
-						logger.Warnf("cannot refresh token. %v", err)
-						credentialsStorage.deleteCredential(newOAuthRefreshTokenSpec(sc.cfg.OauthTokenRequestURL, sc.cfg.User))
-					}
-				}
+				doRefreshTokenWithLock(sc)
 			}
 
 			// if refreshing succeeds for authorization code, we will take a token from cache
@@ -687,7 +794,47 @@ func authenticateWithConfig(sc *snowflakeConn) error {
 			return err
 		}
 	}
+	if sc.cfg.Authenticator == AuthTypeUsernamePasswordMFA && isEligibleForParallelLogin(sc.cfg, sc.cfg.ClientRequestMfaToken) {
+		valueAwaiter := valueAwaitHolder.get(mfaTokenLockKey)
+		valueAwaiter.done()
+	}
+	if sc.cfg.Authenticator == AuthTypeExternalBrowser && isEligibleForParallelLogin(sc.cfg, sc.cfg.ClientStoreTemporaryCredential) {
+		valueAwaiter := valueAwaitHolder.get(idTokenLockKey)
+		valueAwaiter.done()
+	}
 	sc.populateSessionParameters(authData.Parameters)
 	sc.ctx = context.WithValue(sc.ctx, SFSessionIDKey, authData.SessionID)
 	return nil
 }
+
+func doRefreshTokenWithLock(sc *snowflakeConn) {
+	if oauthClient, err := newOauthClient(sc.ctx, sc.cfg, sc); err != nil {
+		logger.Warnf("failed to create oauth client. %v", err)
+	} else {
+		lockKey := newRefreshTokenLockKey(oauthClient.tokenURL(), sc.cfg.User)
+		if _, err = getValueWithLock(chooseLockerForAuth(sc.cfg), lockKey, func() (string, error) {
+			if err = oauthClient.refreshToken(); err != nil {
+				logger.Warnf("cannot refresh token. %v", err)
+				credentialsStorage.deleteCredential(newOAuthRefreshTokenSpec(sc.cfg.OauthTokenRequestURL, sc.cfg.User))
+				return "", err
+			}
+			return "", nil
+		}); err != nil {
+			logger.Warnf("failed to refresh token with lock. %v", err)
+		}
+	}
+}
+
+func chooseLockerForAuth(cfg *Config) locker {
+	if cfg.SingleAuthenticationPrompt == ConfigBoolFalse {
+		return noopLocker
+	}
+	if cfg.User == "" {
+		return noopLocker
+	}
+	return exclusiveLocker
+}
+
+func isEligibleForParallelLogin(cfg *Config, cacheEnabled ConfigBool) bool {
+	return cfg.SingleAuthenticationPrompt != ConfigBoolFalse && cfg.User != "" && cacheEnabled == ConfigBoolTrue
+}
