@@ -2,6 +2,7 @@ package gosnowflake
 
 import (
 	"crypto/rsa"
+	"crypto/tls"
 	"crypto/x509"
 	"encoding/base64"
 	"encoding/pem"
@@ -41,6 +42,17 @@ const (
 	ConfigBoolFalse
 )
 
+func (cb ConfigBool) String() string {
+	switch cb {
+	case ConfigBoolTrue:
+		return "true"
+	case ConfigBoolFalse:
+		return "false"
+	default:
+		return "not set"
+	}
+}
+
 // Config is a set of configuration parameters
 type Config struct {
 	Account   string // Account name
@@ -52,12 +64,13 @@ type Config struct {
 	Role      string // Role
 	Region    string // Region
 
-	OauthClientID         string // Client id for OAuth2 external IdP
-	OauthClientSecret     string // Client secret for OAuth2 external IdP
-	OauthAuthorizationURL string // Authorization URL of Auth2 external IdP
-	OauthTokenRequestURL  string // Token request URL of Auth2 external IdP
-	OauthRedirectURI      string // Redirect URI registered in IdP. The default is http://127.0.0.1:<random port>/
-	OauthScope            string // Comma separated list of scopes. If empty it is derived from role.
+	OauthClientID                string // Client id for OAuth2 external IdP
+	OauthClientSecret            string // Client secret for OAuth2 external IdP
+	OauthAuthorizationURL        string // Authorization URL of Auth2 external IdP
+	OauthTokenRequestURL         string // Token request URL of Auth2 external IdP
+	OauthRedirectURI             string // Redirect URI registered in IdP. The default is http://127.0.0.1:<random port>
+	OauthScope                   string // Comma separated list of scopes. If empty it is derived from role.
+	EnableSingleUseRefreshTokens bool   // Enables single use refresh tokens for Snowflake IdP
 
 	// ValidateDefaultParameters disable the validation checks for Database, Schema, Warehouse and Role
 	// at the time a connection is established
@@ -70,7 +83,8 @@ type Config struct {
 	Host     string // hostname (optional)
 	Port     int    // port (optional)
 
-	Authenticator AuthType // The authenticator type
+	Authenticator              AuthType   // The authenticator type
+	SingleAuthenticationPrompt ConfigBool // If enabled prompting for authentication will only occur for the first authentication challenge
 
 	Passcode           string
 	PasscodeInPassword bool
@@ -100,6 +114,9 @@ type Config struct {
 
 	Transporter http.RoundTripper // RoundTripper to intercept HTTP requests and responses
 
+	tlsConfig     *tls.Config // Custom TLS configuration
+	TLSConfigName string      // Name of the TLS config to use
+
 	DisableTelemetry bool // indicates whether to disable telemetry
 
 	Tracing string // sets logging level
@@ -123,6 +140,22 @@ type Config struct {
 
 	WorkloadIdentityProvider      string // The workload identity provider to use for WIF authentication
 	WorkloadIdentityEntraResource string // The resource to use for WIF authentication on Azure environment
+
+	CertRevocationCheckMode           CertRevocationCheckMode // revocation check mode for CRLs
+	CrlAllowCertificatesWithoutCrlURL ConfigBool              // Allow certificates (not short-lived) without CRL DP included to be treated as correct ones
+	CrlInMemoryCacheDisabled          bool                    // Should the in-memory cache be disabled
+	CrlOnDiskCacheDisabled            bool                    // Should the on-disk cache be disabled
+	CrlHTTPClientTimeout              time.Duration           // Timeout for HTTP client used to download CRL
+
+	ConnectionDiagnosticsEnabled       bool   // Indicates whether connection diagnostics should be enabled
+	ConnectionDiagnosticsAllowlistFile string // File path to the allowlist file for connection diagnostics. If not specified, the allowlist.json file in the current directory will be used.
+
+	ProxyHost     string // Proxy host
+	ProxyPort     int    // Proxy port
+	ProxyUser     string // Proxy user
+	ProxyPassword string // Proxy password
+	ProxyProtocol string // Proxy protocol (http or https)
+	NoProxy       string // No proxy for this host list
 }
 
 // Validate enables testing if config is correct.
@@ -217,6 +250,9 @@ func DSN(cfg *Config) (dsn string, err error) {
 	if cfg.OauthScope != "" {
 		params.Add("oauthScope", cfg.OauthScope)
 	}
+	if cfg.EnableSingleUseRefreshTokens {
+		params.Add("enableSingleUseRefreshTokens", strconv.FormatBool(cfg.EnableSingleUseRefreshTokens))
+	}
 	if cfg.WorkloadIdentityProvider != "" {
 		params.Add("workloadIdentityProvider", cfg.WorkloadIdentityProvider)
 	}
@@ -230,6 +266,13 @@ func DSN(cfg *Config) (dsn string, err error) {
 			params.Add("authenticator", strings.ToLower(cfg.Authenticator.String()))
 		}
 	}
+	if cfg.SingleAuthenticationPrompt != configBoolNotSet {
+		if cfg.SingleAuthenticationPrompt == ConfigBoolTrue {
+			params.Add("singleAuthenticationPrompt", "true")
+		} else {
+			params.Add("singleAuthenticationPrompt", "false")
+		}
+	}
 	if cfg.Passcode != "" {
 		params.Add("passcode", cfg.Passcode)
 	}
@@ -269,6 +312,21 @@ func DSN(cfg *Config) (dsn string, err error) {
 	if cfg.Token != "" {
 		params.Add("token", cfg.Token)
 	}
+	if cfg.CertRevocationCheckMode != CertRevocationCheckDisabled {
+		params.Add("certRevocationCheckMode", cfg.CertRevocationCheckMode.String())
+	}
+	if cfg.CrlAllowCertificatesWithoutCrlURL == ConfigBoolTrue {
+		params.Add("crlAllowCertificatesWithoutCrlURL", "true")
+	}
+	if cfg.CrlInMemoryCacheDisabled {
+		params.Add("crlInMemoryCacheDisabled", "true")
+	}
+	if cfg.CrlOnDiskCacheDisabled {
+		params.Add("crlOnDiskCacheDisabled", "true")
+	}
+	if cfg.CrlHTTPClientTimeout != 0 {
+		params.Add("crlHttpClientTimeout", strconv.FormatInt(int64(cfg.CrlHTTPClientTimeout/time.Second), 10))
+	}
 	if cfg.Params != nil {
 		for k, v := range cfg.Params {
 			params.Add(k, *v)
@@ -288,6 +346,9 @@ func DSN(cfg *Config) (dsn string, err error) {
 	if cfg.DisableOCSPChecks {
 		params.Add("disableOCSPChecks", strconv.FormatBool(cfg.DisableOCSPChecks))
 	}
+	if cfg.DisableTelemetry {
+		params.Add("disableTelemetry", strconv.FormatBool(cfg.DisableTelemetry))
+	}
 	if cfg.Tracing != "" {
 		params.Add("tracing", cfg.Tracing)
 	}
@@ -321,6 +382,33 @@ func DSN(cfg *Config) (dsn string, err error) {
 	if cfg.DisableSamlURLCheck != configBoolNotSet {
 		params.Add("disableSamlURLCheck", strconv.FormatBool(cfg.DisableSamlURLCheck != ConfigBoolFalse))
 	}
+	if cfg.ConnectionDiagnosticsEnabled {
+		params.Add("connectionDiagnosticsEnabled", strconv.FormatBool(cfg.ConnectionDiagnosticsEnabled))
+	}
+	if cfg.ConnectionDiagnosticsAllowlistFile != "" {
+		params.Add("connectionDiagnosticsAllowlistFile", cfg.ConnectionDiagnosticsAllowlistFile)
+	}
+	if cfg.TLSConfigName != "" {
+		params.Add("tlsConfigName", cfg.TLSConfigName)
+	}
+	if cfg.ProxyHost != "" {
+		params.Add("proxyHost", cfg.ProxyHost)
+	}
+	if cfg.ProxyPort != 0 {
+		params.Add("proxyPort", strconv.Itoa(cfg.ProxyPort))
+	}
+	if cfg.ProxyProtocol != "" {
+		params.Add("proxyProtocol", cfg.ProxyProtocol)
+	}
+	if cfg.ProxyUser != "" {
+		params.Add("proxyUser", cfg.ProxyUser)
+	}
+	if cfg.ProxyPassword != "" {
+		params.Add("proxyPassword", cfg.ProxyPassword)
+	}
+	if cfg.NoProxy != "" {
+		params.Add("noProxy", cfg.NoProxy)
+	}
 
 	dsn = fmt.Sprintf("%v:%v@%v:%v", url.QueryEscape(cfg.User), url.QueryEscape(cfg.Password), cfg.Host, cfg.Port)
 	if params.Encode() != "" {
@@ -351,8 +439,8 @@ func ParseDSN(dsn string) (cfg *Config, err error) {
 	var i int
 	posQuestion := len(dsn)
 	for i = len(dsn) - 1; i >= 0; i-- {
-		switch {
-		case dsn[i] == '/':
+		switch dsn[i] {
+		case '/':
 			foundSlash = true
 
 			// left part is empty if i <= 0
@@ -360,12 +448,12 @@ func ParseDSN(dsn string) (cfg *Config, err error) {
 			posSecondSlash := i
 			if i > 0 {
 				for j = i - 1; j >= 0; j-- {
-					switch {
-					case dsn[j] == '/':
+					switch dsn[j] {
+					case '/':
 						// second slash
 						secondSlash = true
 						posSecondSlash = j
-					case dsn[j] == '@':
+					case '@':
 						// username[:password]@...
 						cfg.User, cfg.Password = parseUserPassword(j, dsn)
 					}
@@ -393,7 +481,7 @@ func ParseDSN(dsn string) (cfg *Config, err error) {
 				cfg.Database = dsn[posSecondSlash+1 : posQuestion]
 			}
 			done = true
-		case dsn[i] == '?':
+		case '?':
 			posQuestion = i
 		}
 		if done {
@@ -404,10 +492,10 @@ func ParseDSN(dsn string) (cfg *Config, err error) {
 		// no db or schema is specified
 		var j int
 		for j = len(dsn) - 1; j >= 0; j-- {
-			switch {
-			case dsn[j] == '@':
+			switch dsn[j] {
+			case '@':
 				cfg.User, cfg.Password = parseUserPassword(j, dsn)
-			case dsn[j] == '?':
+			case '?':
 				posQuestion = j
 			}
 			if dsn[j] == '@' {
@@ -570,6 +658,10 @@ func fillMissingConfigParameters(cfg *Config) error {
 		cfg.IncludeRetryReason = ConfigBoolTrue
 	}
 
+	if cfg.ProxyHost != "" && cfg.ProxyProtocol == "" {
+		cfg.ProxyProtocol = "http" // Default to http if not specified
+	}
+
 	domain, _ := extractDomainFromHost(cfg.Host)
 	if len(cfg.Host) == len(domain) {
 		return &SnowflakeError{
@@ -578,6 +670,16 @@ func fillMissingConfigParameters(cfg *Config) error {
 			MessageArgs: []interface{}{cfg.Host},
 		}
 	}
+	if cfg.TLSConfigName != "" {
+		if tlsConfig, ok := getTLSConfig(cfg.TLSConfigName); ok {
+			cfg.tlsConfig = tlsConfig
+		} else {
+			return &SnowflakeError{
+				Number:  ErrCodeMissingTLSConfig,
+				Message: fmt.Sprintf(errMsgMissingTLSConfig, cfg.TLSConfigName),
+			}
+		}
+	}
 	return nil
 }
 
@@ -715,7 +817,7 @@ func parseParams(cfg *Config, posQuestion int, dsn string) (err error) {
 
 // parseDSNParams parses the DSN "query string". Values must be url.QueryEscape'ed
 func parseDSNParams(cfg *Config, params string) (err error) {
-	logger.Infof("Query String: %v\n", params)
+	logger.Infof("Query String: %v\n", maskSecrets(params))
 	paramsSlice := strings.Split(params, "&")
 	insecureModeIdx := findByPrefix(paramsSlice, "insecureMode")
 	disableOCSPChecksIdx := findByPrefix(paramsSlice, "disableOCSPChecks")
@@ -749,6 +851,17 @@ func parseDSNParams(cfg *Config, params string) (err error) {
 			cfg.Region = value
 		case "protocol":
 			cfg.Protocol = value
+		case "singleAuthenticationPrompt":
+			var vv bool
+			vv, err = strconv.ParseBool(value)
+			if err != nil {
+				return
+			}
+			if vv {
+				cfg.SingleAuthenticationPrompt = ConfigBoolTrue
+			} else {
+				cfg.SingleAuthenticationPrompt = ConfigBoolFalse
+			}
 		case "passcode":
 			cfg.Passcode = value
 		case "oauthClientId":
@@ -763,6 +876,13 @@ func parseDSNParams(cfg *Config, params string) (err error) {
 			cfg.OauthRedirectURI = value
 		case "oauthScope":
 			cfg.OauthScope = value
+		case "enableSingleUseRefreshTokens":
+			var vv bool
+			vv, err = strconv.ParseBool(value)
+			if err != nil {
+				return
+			}
+			cfg.EnableSingleUseRefreshTokens = vv
 		case "passcodeInPassword":
 			var vv bool
 			vv, err = strconv.ParseBool(value)
@@ -832,6 +952,13 @@ func parseDSNParams(cfg *Config, params string) (err error) {
 				return
 			}
 			cfg.DisableOCSPChecks = vv
+		case "disableTelemetry":
+			var vv bool
+			vv, err = strconv.ParseBool(value)
+			if err != nil {
+				return
+			}
+			cfg.DisableTelemetry = vv
 		case "ocspFailOpen":
 			var vv bool
 			vv, err = strconv.ParseBool(value)
@@ -846,6 +973,8 @@ func parseDSNParams(cfg *Config, params string) (err error) {
 
 		case "token":
 			cfg.Token = value
+		case "tlsConfigName":
+			cfg.TLSConfigName = value
 		case "workloadIdentityProvider":
 			cfg.WorkloadIdentityProvider = value
 		case "workloadIdentityEntraResource":
@@ -943,6 +1072,71 @@ func parseDSNParams(cfg *Config, params string) (err error) {
 			} else {
 				cfg.DisableSamlURLCheck = ConfigBoolFalse
 			}
+		case "certRevocationCheckMode":
+			var certRevocationCheckMode CertRevocationCheckMode
+			certRevocationCheckMode, err = parseCertRevocationCheckMode(value)
+			if err != nil {
+				return
+			}
+			cfg.CertRevocationCheckMode = certRevocationCheckMode
+		case "crlAllowCertificatesWithoutCrlURL":
+			var vv bool
+			vv, err = strconv.ParseBool(value)
+			if vv {
+				cfg.CrlAllowCertificatesWithoutCrlURL = ConfigBoolTrue
+			} else {
+				cfg.CrlAllowCertificatesWithoutCrlURL = ConfigBoolFalse
+			}
+		case "crlInMemoryCacheDisabled":
+			var vv bool
+			vv, err = strconv.ParseBool(value)
+			if err != nil {
+				return
+			}
+			if vv {
+				cfg.CrlInMemoryCacheDisabled = true
+			} else {
+				cfg.CrlInMemoryCacheDisabled = false
+			}
+		case "crlOnDiskCacheDisabled":
+			var vv bool
+			vv, err = strconv.ParseBool(value)
+			if err != nil {
+				return
+			}
+			if vv {
+				cfg.CrlOnDiskCacheDisabled = true
+			} else {
+				cfg.CrlOnDiskCacheDisabled = false
+			}
+		case "crlHttpClientTimeout":
+			var vv int64
+			vv, err = strconv.ParseInt(value, 10, 64)
+			if err != nil {
+				return
+			}
+			cfg.CrlHTTPClientTimeout = time.Duration(vv * int64(time.Second))
+		case "connectionDiagnosticsEnabled":
+			var vv bool
+			vv, err = strconv.ParseBool(value)
+			if err != nil {
+				return
+			}
+			cfg.ConnectionDiagnosticsEnabled = vv
+		case "connectionDiagnosticsAllowlistFile":
+			cfg.ConnectionDiagnosticsAllowlistFile = value
+		case "proxyHost":
+			cfg.ProxyHost, err = parseString(value)
+		case "proxyPort":
+			cfg.ProxyPort, err = parseInt(value)
+		case "proxyUser":
+			cfg.ProxyUser, err = parseString(value)
+		case "proxyPassword":
+			cfg.ProxyPassword, err = parseString(value)
+		case "noProxy":
+			cfg.NoProxy, err = parseString(value)
+		case "proxyProtocol":
+			cfg.ProxyProtocol, err = parseString(value)
 		default:
 			if cfg.Params == nil {
 				cfg.Params = make(map[string]*string)
