@@ -237,7 +237,11 @@ func postRestfulQueryHelper(
 	if err != nil {
 		return nil, err
 	}
-	defer resp.Body.Close()
+	defer func() {
+		if closeErr := resp.Body.Close(); closeErr != nil {
+			logger.WithContext(ctx).Warnf("failed to close response body for %v. err: %v", fullURL, closeErr)
+		}
+	}()
 
 	if resp.StatusCode == http.StatusOK {
 		logger.WithContext(ctx).Infof("postQuery: resp: %v", resp)
@@ -280,9 +284,13 @@ func postRestfulQueryHelper(
 				logger.WithContext(ctx).Errorf("failed to get response. err: %v", err)
 				return nil, err
 			}
+			defer func() {
+				if closeErr := resp.Body.Close(); closeErr != nil {
+					logger.WithContext(ctx).Warnf("failed to close response body for %v. err: %v", fullURL, closeErr)
+				}
+			}()
 			respd = execResponse{} // reset the response
 			err = json.NewDecoder(resp.Body).Decode(&respd)
-			resp.Body.Close()
 			if err != nil {
 				logger.WithContext(ctx).Errorf("failed to decode JSON. err: %v", err)
 				return nil, err
@@ -329,7 +337,11 @@ func closeSession(ctx context.Context, sr *snowflakeRestful, timeout time.Durati
 	if err != nil {
 		return err
 	}
-	defer resp.Body.Close()
+	defer func() {
+		if err = resp.Body.Close(); err != nil {
+			logger.WithContext(ctx).Warnf("failed to close response body for %v. err: %v", fullURL, err)
+		}
+	}()
 	if resp.StatusCode == http.StatusOK {
 		var respd renewSessionResponse
 		if err = json.NewDecoder(resp.Body).Decode(&respd); err != nil {
@@ -364,13 +376,12 @@ func closeSession(ctx context.Context, sr *snowflakeRestful, timeout time.Durati
 }
 
 func renewRestfulSession(ctx context.Context, sr *snowflakeRestful, timeout time.Duration) error {
-	logger.WithContext(ctx).Info("start renew session")
 	params := &url.Values{}
 	params.Set(requestIDKey, getOrGenerateRequestIDFromContext(ctx).String())
 	params.Set(requestGUIDKey, NewUUID().String())
 	fullURL := sr.getFullURL(tokenRequestPath, params)
 
-	token, masterToken, _ := sr.TokenAccessor.GetTokens()
+	token, masterToken, sessionID := sr.TokenAccessor.GetTokens()
 	headers := getHeaders()
 	headers[headerAuthorizationKey] = fmt.Sprintf(headerSnowflakeToken, masterToken)
 
@@ -378,6 +389,8 @@ func renewRestfulSession(ctx context.Context, sr *snowflakeRestful, timeout time
 	body["oldSessionToken"] = token
 	body["requestType"] = "RENEW"
 
+	ctx = context.WithValue(ctx, SFSessionIDKey, sessionID)
+	logger.WithContext(ctx).Info("start renew session")
 	var reqBody []byte
 	reqBody, err := json.Marshal(body)
 	if err != nil {
@@ -388,7 +401,11 @@ func renewRestfulSession(ctx context.Context, sr *snowflakeRestful, timeout time
 	if err != nil {
 		return err
 	}
-	defer resp.Body.Close()
+	defer func() {
+		if err = resp.Body.Close(); err != nil {
+			logger.WithContext(ctx).Warnf("failed to close response body for %v. err: %v", fullURL, err)
+		}
+	}()
 	if resp.StatusCode == http.StatusOK {
 		var respd renewSessionResponse
 		err = json.NewDecoder(resp.Body).Decode(&respd)
@@ -407,6 +424,7 @@ func renewRestfulSession(ctx context.Context, sr *snowflakeRestful, timeout time
 			}
 		}
 		sr.TokenAccessor.SetTokens(respd.Data.SessionToken, respd.Data.MasterToken, respd.Data.SessionID)
+		logger.WithContext(ctx).Info("successfully renewed session")
 		return nil
 	}
 	b, err := io.ReadAll(resp.Body)
@@ -460,7 +478,11 @@ func cancelQuery(ctx context.Context, sr *snowflakeRestful, requestID UUID, time
 	if err != nil {
 		return err
 	}
-	defer resp.Body.Close()
+	defer func() {
+		if err = resp.Body.Close(); err != nil {
+			logger.WithContext(ctx).Warnf("failed to close response body for %v. err: %v", fullURL, err)
+		}
+	}()
 	if resp.StatusCode == http.StatusOK {
 		var respd cancelQueryResponse
 		if err = json.NewDecoder(resp.Body).Decode(&respd); err != nil {
@@ -473,8 +495,14 @@ func cancelQuery(ctx context.Context, sr *snowflakeRestful, requestID UUID, time
 				return err
 			}
 			return sr.FuncCancelQuery(ctx, sr, requestID, timeout)
-		} else if !respd.Success && respd.Code == queryNotExecutingCode && ctxRetry != 0 {
-			return sr.FuncCancelQuery(context.WithValue(ctx, cancelRetry, ctxRetry-1), sr, requestID, timeout)
+		} else if !respd.Success && respd.Code == queryNotExecutingCode {
+			if ctxRetry != 0 {
+				return sr.FuncCancelQuery(context.WithValue(ctx, cancelRetry, ctxRetry-1), sr, requestID, timeout)
+			}
+			// After exhausting retries, we can safely treat queryNotExecutingCode as success
+			// since it indicates the query has already completed and there's nothing left to cancel
+			logger.WithContext(ctx).Info("query has already completed, no cancellation needed")
+			return nil
 		} else if respd.Success {
 			return nil
 		} else {
