@@ -1,4 +1,4 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright IBM Corp. 2016, 2025
 // SPDX-License-Identifier: BUSL-1.1
 
 package cert
@@ -435,6 +435,7 @@ func (b *backend) matchesConstraints(ctx context.Context, clientCert *x509.Certi
 		b.matchesEmailSANs(clientCert, config) &&
 		b.matchesURISANs(clientCert, config) &&
 		b.matchesOrganizationalUnits(clientCert, config) &&
+		b.matchesOrganizations(clientCert, config) &&
 		b.matchesCertificateExtensions(clientCert, config)
 	if config.Entry.OcspEnabled {
 		ocspGood, err := b.checkForCertInOCSP(ctx, clientCert, trustedChain, conf)
@@ -549,7 +550,7 @@ func (b *backend) matchesURISANs(clientCert *x509.Certificate, config *ParsedCer
 	return false
 }
 
-// matchesOrganizationalUnits verifies that the certificate matches at least one configurd allowed OU
+// matchesOrganizationalUnits verifies that the certificate matches at least one configured allowed OU
 func (b *backend) matchesOrganizationalUnits(clientCert *x509.Certificate, config *ParsedCert) bool {
 	// Default behavior (no OUs) is to allow all OUs
 	if len(config.Entry.AllowedOrganizationalUnits) == 0 {
@@ -568,6 +569,25 @@ func (b *backend) matchesOrganizationalUnits(clientCert *x509.Certificate, confi
 	return false
 }
 
+// matchesOrganizations verifies that the certificate matches at least one configured allowed organization (O)
+func (b *backend) matchesOrganizations(clientCert *x509.Certificate, config *ParsedCert) bool {
+	// Default behavior (no Organizations (Os)) is to allow all organizations (Os)
+	if len(config.Entry.AllowedOrganizations) == 0 {
+		return true
+	}
+
+	// At least one pattern must match at least one name if any patterns are specified
+	for _, allowedOrganization := range config.Entry.AllowedOrganizations {
+		for _, o := range clientCert.Subject.Organization {
+			if glob.Glob(allowedOrganization, o) {
+				return true
+			}
+		}
+	}
+
+	return false
+}
+
 // matchesCertificateExtensions verifies that the certificate matches configured
 // required extensions
 func (b *backend) matchesCertificateExtensions(clientCert *x509.Certificate, config *ParsedCert) bool {
