@@ -1,4 +1,4 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright IBM Corp. 2016, 2025
 // SPDX-License-Identifier: BUSL-1.1
 
 package pki
@@ -17,6 +17,7 @@ import (
 	"github.com/hashicorp/go-secure-stdlib/parseutil"
 	"github.com/hashicorp/vault/builtin/logical/pki/issuing"
 	"github.com/hashicorp/vault/builtin/logical/pki/managed_key"
+	"github.com/hashicorp/vault/builtin/logical/pki/observe"
 	"github.com/hashicorp/vault/builtin/logical/pki/pki_backend"
 	"github.com/hashicorp/vault/builtin/logical/pki/revocation"
 	"github.com/hashicorp/vault/helper/metricsutil"
@@ -311,9 +312,13 @@ func Backend(conf *logical.BackendConfig) *backend {
 	b.acmeState = NewACMEState()
 	b.certificateCounter = NewCertificateCounter(b.backendUUID)
 
+	b.pkiCertificateCounter = logical.NewNullPkiCertificateCounter()
+
+	b.pkiObserver = observe.NewPkiCeObserver(b.Logger(), b)
+
 	// It is important that we call SetupEnt at the very end as
 	// some ENT backends need access to the member vars initialized above.
-	b.SetupEnt()
+	b.SetupEnt(conf)
 	return &b
 }
 
@@ -340,8 +345,13 @@ type backend struct {
 
 	unifiedTransferStatus *UnifiedTransferStatus
 
+	// certificateCounter emits metrics about the number of stored certificates.
 	certificateCounter *CertificateCounter
 
+	// pkiCertificateCounter keeps track of issued and stored certificate counts
+	// for the purposes of PKI-only billing.
+	pkiCertificateCounter logical.PkiCertificateCounter
+
 	pkiStorageVersion atomic.Value
 	crlBuilder        *CrlBuilder
 
@@ -354,6 +364,8 @@ type backend struct {
 
 	// Track when this mount was started.
 	mountStartup time.Time
+
+	pkiObserver observe.PkiObserver
 }
 
 // BackendOps a bridge/legacy interface until we can further
