@@ -214,7 +214,8 @@ func (b *backend) pathGenerateIntermediate(ctx context.Context, req *logical.Req
 	b.pkiObserver.RecordPKIObservation(ctx, req, observe.ObservationTypePKIGenerateIntermediate,
 		observe.NewAdditionalPKIMetadata("key_id", myKey.ID),
 		observe.NewAdditionalPKIMetadata("key_name", myKey.Name),
-		observe.NewAdditionalPKIMetadata("role", role.Name),
+		observe.NewAdditionalPKIMetadata("key_type", myKey.PrivateKeyType),
+		observe.NewAdditionalPKIMetadata("role_name", role.Name),
 		observe.NewAdditionalPKIMetadata("exported", exported),
 		observe.NewAdditionalPKIMetadata("type", format))
 
