@@ -1,4 +1,4 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright IBM Corp. 2016, 2025
 // SPDX-License-Identifier: BUSL-1.1
 
 package pki
@@ -17,6 +17,7 @@ import (
 	"github.com/hashicorp/go-hclog"
 	"github.com/hashicorp/go-secure-stdlib/parseutil"
 	"github.com/hashicorp/vault/builtin/logical/pki/issuing"
+	"github.com/hashicorp/vault/builtin/logical/pki/observe"
 	"github.com/hashicorp/vault/builtin/logical/pki/revocation"
 	"github.com/hashicorp/vault/helper/constants"
 	"github.com/hashicorp/vault/sdk/framework"
@@ -705,6 +706,14 @@ func (b *backend) pathTidyWrite(ctx context.Context, req *logical.Request, d *fr
 		}
 	}
 
+	b.pkiObserver.RecordPKIObservation(ctx, req, observe.ObservationTypePKITidy,
+		observe.NewAdditionalPKIMetadata("tidy_cert_store", tidyCertStore),
+		observe.NewAdditionalPKIMetadata("tidy_revoked_certs", tidyRevokedCerts),
+		observe.NewAdditionalPKIMetadata("tidy_revoked_cert_issuer_associations", tidyRevokedAssocs),
+		observe.NewAdditionalPKIMetadata("tidy_expired_issuers", tidyExpiredIssuers),
+		observe.NewAdditionalPKIMetadata("tidy_backup_bundle", tidyBackupBundle),
+	)
+
 	return logical.RespondWithStatusCode(resp, req, http.StatusAccepted)
 }
 
