@@ -1,4 +1,4 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright IBM Corp. 2016, 2025
 // SPDX-License-Identifier: BUSL-1.1
 
 package configutil
@@ -167,6 +167,10 @@ type Listener struct {
 	// CustomMaxJSONArrayElementCount determines the maximum number of elements in a JSON array.
 	CustomMaxJSONArrayElementCountRaw interface{} `hcl:"max_json_array_element_count"`
 	CustomMaxJSONArrayElementCount    int64       `hcl:"-"`
+
+	// CustomMaxJSONToken determines the maximum number of tokens in a JSON.
+	CustomMaxJSONTokenRaw interface{} `hcl:"max_json_token"`
+	CustomMaxJSONToken    int64       `hcl:"-"`
 }
 
 // AgentAPI allows users to select which parts of the Agent API they want enabled.
@@ -762,5 +766,12 @@ func (l *Listener) parseJSONLimitsSettings() error {
 		return fmt.Errorf("max_json_array_element_count cannot be negative")
 	}
 
+	if err := parseAndClearInt(&l.CustomMaxJSONTokenRaw, &l.CustomMaxJSONToken); err != nil {
+		return fmt.Errorf("error parsing max_json_token: %w", err)
+	}
+	if l.CustomMaxJSONToken < 0 {
+		return fmt.Errorf("max_json_token cannot be negative")
+	}
+
 	return nil
 }
