@@ -410,10 +410,13 @@ func (c *Core) emitMetricsActiveNode(stopCh chan struct{}) {
 }
 
 type kvMount struct {
-	Namespace  *namespace.Namespace
-	MountPoint string
-	Version    string
-	NumSecrets int
+	Namespace            *namespace.Namespace
+	MountPoint           string
+	MountAccessor        string
+	Version              string
+	Local                bool
+	NumSecrets           int
+	RunningPluginVersion string
 }
 
 func (c *Core) findKvMounts() []*kvMount {
@@ -436,10 +439,13 @@ func (c *Core) findKvMounts() []*kvMount {
 				version = "1"
 			}
 			mounts = append(mounts, &kvMount{
-				Namespace:  entry.namespace,
-				MountPoint: entry.Path,
-				Version:    version,
-				NumSecrets: 0,
+				Namespace:            entry.namespace,
+				MountPoint:           entry.Path,
+				MountAccessor:        entry.Accessor,
+				Version:              version,
+				NumSecrets:           0,
+				Local:                entry.Local,
+				RunningPluginVersion: entry.RunningVersion,
 			})
 		}
 	}
