@@ -1,4 +1,4 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright IBM Corp. 2016, 2025
 // SPDX-License-Identifier: BUSL-1.1
 
 package vault
@@ -599,6 +599,9 @@ func (r *Router) routeCommon(ctx context.Context, req *logical.Request, existenc
 	req.Path = adjustedPath
 	if !existenceCheck {
 		metricName := []string{"route", string(req.Operation)}
+		if req.IsSnapshotReadOrList() {
+			metricName[1] = metricName[1] + "-snapshot"
+		}
 		if req.Operation != logical.RollbackOperation || r.rollbackMetricsMountName {
 			metricName = append(metricName, strings.ReplaceAll(mount, "/", "-"))
 		}
@@ -634,6 +637,10 @@ func (r *Router) routeCommon(ctx context.Context, req *logical.Request, existenc
 	// Adjust the path to exclude the routing prefix
 	originalPath := req.Path
 	req.Path = strings.TrimPrefix(ns.Path+req.Path, mount)
+	originalRecoverSourcePath := req.RecoverSourcePath
+	if req.RecoverSourcePath != "" {
+		req.RecoverSourcePath = strings.TrimPrefix(ns.Path+req.RecoverSourcePath, mount)
+	}
 	req.MountPoint = mount
 	req.MountType = re.mountEntry.Type
 	req.SetMountRunningSha256(re.mountEntry.RunningSha256)
@@ -754,6 +761,7 @@ func (r *Router) routeCommon(ctx context.Context, req *logical.Request, existenc
 	// Reset the request before returning
 	defer func() {
 		req.Path = originalPath
+		req.RecoverSourcePath = originalRecoverSourcePath
 		req.MountPoint = mount
 		req.MountType = re.mountEntry.Type
 		req.SetMountRunningSha256(re.mountEntry.RunningSha256)
